const PCS_DARK    = '(prefers-color-scheme: dark)'; /* медиа-запрос
                                 соответствия тёмной теме */
const STORAGE_KEY = "theme"; /* ключ для хранения настроек, он же
                                 и имя атрибута для <html> */
const THEME_NAME  = ['dark', 'light']; /* возможные названия тем */


/* Определяет, является ли текущая тема в системе "тёмной"? */
const isSystemDarkTheme = () => (window.matchMedia(PCS_DARK).matches);

/* Определяет, необходимо ли пользователю показывать светлую тему? */
const isLightTheme = () => {
	const prefTheme = window.localStorage.getItem(STORAGE_KEY);
	if (typeof prefTheme === 'string')
		return (prefTheme == THEME_NAME[+true]);

	return !isSystemDarkTheme();
}

/* Устанавливает тему в соответствии с текущими настройками пользователя
   либо системы */
const syncTheme = () => {
	const MarkSR = 'marked-source';
    
	isLight = isLightTheme(); /* необходима ли светлая тема? */

	/* в элемент <html> будет добавлен аттрибут theme=dark,
		если необходимо установить тёмную тему, либо он будет
		удален если необходимо установить светлую тему */
	if (isLight)
		document.documentElement.removeAttribute(STORAGE_KEY);
	else
		document.documentElement.setAttribute(STORAGE_KEY, THEME_NAME[+isLight]);

	/* Удалим все источники внутри <picture> который добавил
		данный код, они помечены атрибутом с именем записанным
		в константе MarkSR */
	document
		.querySelectorAll(`picture > source[${MarkSR}]`)
		.forEach((el) => el.remove());

	/* Найдем каждый <source> внутри <picture> у которого
		медиа-запрос соответсвует той теме, которую мы сейчас
		хотим установить */
	document
		.querySelectorAll(`picture > source[media*="(prefers-color-scheme: ${THEME_NAME[+isLight]})"]`)
		.forEach((el) => {                /* ...каждый такой источник */
			const cloned = el.cloneNode();   /* клонируем */
			cloned.removeAttribute("media"); /* удаляем у него атрибут media */
			cloned.setAttribute(MarkSR, ''); /* помечаем его */
			el.parentNode.prepend(cloned);   /* добавляем в самое начало */
		});

	/* Установка цвета заголовка окна */
	var style = getComputedStyle(document.documentElement);
	let themeMeta = document.querySelector('meta[name="theme-color"]');
	if (themeMeta) {
		themeMeta.content = style.getPropertyValue('--title-color');
	}
}

/* Принудительно устанавливает для сайта заданную тему, в качестве
   аргумента передается true для светлой и false для тёмной темы */
const setTheme = (isLight) => {
	window.localStorage.setItem(STORAGE_KEY, THEME_NAME[+isLight]);
	syncTheme();
}

syncTheme();

window.onload = () => {
	syncTheme();
}

window
	.matchMedia(PCS_DARK)
	.addEventListener('change', ({matches:isDark}) => setTheme(!isDark))